/*
* DOGM163 Software	
* This is free software: you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* DogLcd is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with DogLcd. If not, see <http://www.gnu.org/licenses/>.
*
* Copyright 2010 Eberhard Fahle <e.fahle@wayoda.org>
*/

/*
* ATMEGA644
* Quarz 11059200 MHZ
* DOGM163 Test kein echtes SPI, jeder Port verwendbar!  3,3 Volt
* Anschaltung Display in der dogm163.h
* LCD_BACKLIGHT funktioniert in dieser Form nur auf dem Atmega644
* printf("xxx") eingebunden
* Quellcode fr DOGM163 bernommen, siehe Lizenz oben
*/

#include "tools.h"

int dogm_putchar(char c, FILE *stream);
void rv3029_schreiben(void);
void rv3029_lesen(void);
char rtc_bcd_to_hex(char v);
char rtc_hex_to_bcd(char v);
void ausgabe(void);

#define SLAVE_ADRESSE 0xAC


// hier liegen die aktuellen Daten fr Ausgabe und usw.	
// z.B. Stunde  if(rtc_buf.stunde == 24 ){ tu was  }
// die Daten knnen aus jeder Funktion aufgerufen werden					
//
					
typedef struct { 			//  -------- Struktur ----------- fr globale Daten
	uint8_t control_0;		//C0 		0x00
	uint8_t control_1;		//C1
	uint8_t control_2;		//C2
	uint8_t control_3;		//C3
	uint8_t control_4;		//C4
	uint8_t sekunde;		//sek 		0x08
	uint8_t minute;			//min
	uint8_t stunde;			//std
	uint8_t monat;			//monat
	uint8_t tag	;			//tag
	uint8_t w_tag;			//w_tag
	uint8_t jahr;			//jahr		0x0E
	uint8_t sekunde_al;		//sek_al	0x10
	uint8_t minute_al;		//min_al
	uint8_t stunde_al;		//std_al
	uint8_t tag_al	;		//tag_al
	uint8_t w_tag_al;		//w_tag_al
	uint8_t monat_al;		//monat_al
	uint8_t jahr_al;		//jahr_al	0x16
	uint8_t temp;		    //Temp		0x20
	} MY_BUFFER;

	static MY_BUFFER rtc_buf; 

static FILE mystdout = FDEV_SETUP_STREAM(dogm_putchar, NULL, _FDEV_SETUP_WRITE); // bindet printf() ein

int main(void)
{
	stdout = &mystdout; // bindet printf() ein 
	DDRC = 0x03; //SCL/SDA Out
	i2c_init();  
 	lcd_setbacklight(0x88);
 	lcd_init(); 
	lcd_clear();	
 	lcd_set_contrast(0x27);	
	
	//********* nur einmal zur Einstellung starten, dann mit // ausschalten ******
	rv3029_schreiben(); 


while (1)
{
	

	rv3029_lesen();
	_delay_ms(25);
	_delay_ms(25);
	ausgabe();

}


		return (0);
}

void ausgabe(void) 	// RTC Uhr RV3209CS auslesen und auf Display ausgeben, 
				
	{
 
	lcd_set_cursor(0,1);
	printf("%02d.%02d.%04d",rtc_buf.tag,rtc_buf.monat,rtc_buf.jahr+2000); 	// Ausgabe Tag,Monat,Jahr
 	lcd_set_cursor(0,2);
	printf(" %02d:%02d:%02d",rtc_buf.stunde,rtc_buf.minute,rtc_buf.sekunde);// Ausgabe Stunde, Minute, Sekunde
	
	}


void rv3029_schreiben(void)
{
   
		//// REG0 bis REG4  siehe Datenblatt

    	i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);   // set device address and write mode
  		
		i2c_write(0x00);       // Startadresse
		i2c_write(0xE9);       // REG0 //Clock Output pin enable, Enables 1Hz Clock Source for Watch
		i2c_write(0x00);       // REG1
		i2c_write(0x00);       // REG2
		i2c_write(0x00);       // REG3
		i2c_write(0x00);       // REG4

		i2c_stop();                            // Bus stop
	
		
		// EEPROM CONTROL

  		i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);  // set device address and write mode
    		
		i2c_write(0x30); // Startadresse
		i2c_write(0x0F); // EEPROM CONTROL (address 30h) FD0 FD1 ThE ThP  = 0x0F 
						 //	Selects Clock Frequency at CLKOUT pin, 0,5 Sek 
		i2c_stop(); 	 //	Enables Thermometer,
						 // set Temperature Scanning Interval: 16 seconds

		
		// Datum/Uhrzeit senden an RTC
  		
		i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);       // set device address and write mode
    		
		
		i2c_write(0x08); // Startadresse
		
		i2c_write(0x00); // Sekunden
		
		i2c_write(0x00); // Minuten
		
		i2c_write(0x20); // Stunden  
		
		i2c_write(0x03); // Tag  
		
		i2c_write(0x03); // Tag in der Woche    
		
		i2c_write(0x05); // Monat  
		
		i2c_write(0x11); // Jahr 
		
		
		i2c_stop();      // Bus stop
	



}

//--------------------------------lesen--------------------------		
void rv3029_lesen(void)
{
 
	i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);   // set device address and write mode
 	i2c_write(0x08);  	// Startadresse fr Ausgabe Zeit/Datum
	i2c_stop(); 		// Bus stop
		                           
	
    i2c_rep_start(SLAVE_ADRESSE+I2C_READ);       // set device address and write mode
	
		
	rtc_buf.sekunde  = rtc_bcd_to_hex(i2c_readAck())& 0x7f; 	//sek
	rtc_buf.minute   = rtc_bcd_to_hex(i2c_readAck())& 0x7f; 	//min
	rtc_buf.stunde   = rtc_bcd_to_hex(i2c_readAck())& 0x3f; 	//std 
	rtc_buf.tag      = rtc_bcd_to_hex(i2c_readAck())& 0x3f; 	//tag
	rtc_buf.w_tag    = rtc_bcd_to_hex(i2c_readAck())& 0x07; 	//tag in der Woche
	rtc_buf.monat    = rtc_bcd_to_hex(i2c_readAck())& 0x1f; 	//monat
	rtc_buf.jahr     = rtc_bcd_to_hex(i2c_readNak())& 0x1f; 	//jahr



return;
}




int dogm_putchar(char c, FILE *stream) // // bindet printf() ein
{
 	lcd_write_char(c); //LIB DOGS102
    return 0;
}




char rtc_bcd_to_hex(char v)// rtc_bcd_to_hex: wandelt BCD zu HEX Zahl um 
{
  char n;

  n=v; 
  n=n & 0xF0;
  n=n/16;
  v =v & 0x0F;
  n=n*10+v;
  return(n);
}

char rtc_hex_to_bcd(char v)// rtc_hex_to_bcd: wandelt HEX zu BCD Zahl um
{
  char n;

  n=v; 
  n=n/10;
  n=n*16;
  v =v % 10;
  n=n+v;
  return(n);
}


